/**
  ******************************************************************************
  * @file      ex_rtc.c
  * @author    Marek Laban, Marcel Kleja, MICRONIC a.s., LabHC
  * @version   v1.0
  * @note      Created in frame of HECTOR project (No. 644052).
  * @copyright MICRONIC a.s.,
  * @date      11-December-2016
  * @revision  v1.1 (ML)
  *             - Initial version
  * @brief     Fundamental functions for communication with external RTC timer
  *
  ******************************************************************************
  */

#include <stdio.h>
#include "drivers/mss_i2c/mss_i2c.h"
#include "ex_rtc.h"
#include "FatFs/src/integer.h"
#include "drivers/mss_uart/mss_uart.h"
#include "time.h"

/* I2C address of RTC */
#define RTC_ADDR   (0xD0 >> 1)

/* Size of buffer used in rtc functions */
#define RTC_BUFF_SIZE 9

/* I2C timeout used in RTC functions */
#define RTC_TIMEOUT 1000

extern DWORD get_fattime(void);
static uint8_t rtc_state = RTC_NREADY;


/**
  * @brief  Initialization function of RTC
  *            - Initialize of I2C bus
  *            - Wake up RTC timer
  * @param  None
  * @retval None
  */
void init_rtc(void){
  uint8_t rtc_buff[RTC_BUFF_SIZE];
  mss_i2c_status_t i2c_status;
  MSS_I2C_init( &g_mss_i2c0, RTC_ADDR, MSS_I2C_PCLK_DIV_160 );

  /* Halt bit (HB) clear */
  rtc_buff[0] = 0x0C;      // HB address
  rtc_buff[1] = 0x00;      // register value
  MSS_I2C_write(&g_mss_i2c0, RTC_ADDR, rtc_buff, 2, MSS_I2C_RELEASE_BUS);

  /* Wait while I2C operation is finished */
  i2c_status = MSS_I2C_wait_complete( &g_mss_i2c0, RTC_TIMEOUT);
  if (i2c_status ==  MSS_I2C_SUCCESS)
    rtc_state = RTC_READY;
  else
	  rtc_state = RTC_NREADY;

}


/**
  * @brief  get time of external RTC
  * @param[out]  rtc_array - address with configuration parameter of RTC,
  *                          array must have 6 bytes as follows
  *             rtc_array[RTC_YR] - year
  *             rtc_array[RTC_MH] - month
  *             rtc_array[RTC_DY] - day
  *             rtc_array[RTC_HR] - hour
  *             rtc_array[RTC_MT] - minute
  *             rtc_array[RTC_SC] - second
  *
  * @retval None
  */
void get_rtc_time (uint8_t rtc_array[RTC_SIZE]){
  uint8_t rtc_buff[RTC_BUFF_SIZE];

  /* set init address and read whole RTC memory*/
  rtc_buff[0] = 0x00;
  MSS_I2C_write(&g_mss_i2c0, RTC_ADDR, rtc_buff, 1, MSS_I2C_RELEASE_BUS);
  MSS_I2C_wait_complete( &g_mss_i2c0, RTC_TIMEOUT );

  /* read time data */
  MSS_I2C_read(&g_mss_i2c0, RTC_ADDR, rtc_buff, RTC_BUFF_SIZE, MSS_I2C_RELEASE_BUS);
  MSS_I2C_wait_complete( &g_mss_i2c0, RTC_TIMEOUT );

  rtc_array[RTC_SC] = (uint8_t)((rtc_buff [1] & 0x0F) + 10 * ((rtc_buff [1] & 0x70) >> 4));
  rtc_array[RTC_MI] = (uint8_t)((rtc_buff [2] & 0x0F) + 10 * ((rtc_buff [2] & 0x70) >> 4));
  rtc_array[RTC_HR] = (uint8_t)((rtc_buff [3] & 0x0F) + 10 * ((rtc_buff [3] & 0x30) >> 4));
  rtc_array[RTC_DY] = (uint8_t)((rtc_buff [5] & 0x0F) + 10 * ((rtc_buff [5] & 0x30) >> 4));
  rtc_array[RTC_MO] = (uint8_t)((rtc_buff [6] & 0x0F) + 10 * ((rtc_buff [6] & 0x10) >> 4));
  rtc_array[RTC_YR] = (uint8_t)((rtc_buff [7] & 0x0F) + 10 * ((rtc_buff [7] & 0xF0) >> 4));


}


/**
  * @brief  Set time of external RTC
  * @param[in]  rtc_array - address with configuration parameter of RTC,
  *                         array must have 6 bytes as follows
  *             rtc_array[RTC_YR] - year
  *             rtc_array[RTC_MH] - month
  *             rtc_array[RTC_DY] - day
  *             rtc_array[RTC_HR] - hour
  *             rtc_array[RTC_MT] - minute
  *             rtc_array[RTC_SC] - second
  *
  * @retval None
  */
void set_rtc_time (uint8_t rtc_array[RTC_SIZE]){

  uint8_t rtc_buff[RTC_BUFF_SIZE];
  /* set init address */
  rtc_buff[0]              = RTC_REG_MS;          // set initial address
  rtc_buff[1 + RTC_REG_MS] = 0;                   // miliseconds
  rtc_buff[1 + RTC_REG_SC] = (uint8_t)0x7F & ((rtc_array[RTC_SC] % 10) | ((rtc_array[RTC_SC] / 10) << 4));   // seconds
  rtc_buff[1 + RTC_REG_MI] = (uint8_t)0x7F & ((rtc_array[RTC_MI] % 10) | ((rtc_array[RTC_MI] / 10) << 4));   // minutes
  rtc_buff[1 + RTC_REG_HR] = (uint8_t)0x3F & ((rtc_array[RTC_HR] % 10) | ((rtc_array[RTC_HR] / 10) << 4));   // hours
  rtc_buff[1 + RTC_REG_DW] = (uint8_t)0x00;       // day of week
  rtc_buff[1 + RTC_REG_DY] = (uint8_t)0x3F & ((rtc_array[RTC_DY] % 10) | ((rtc_array[RTC_DY] / 10) << 4));   // day of month
  rtc_buff[1 + RTC_REG_MO] = (uint8_t)0x1F & ((rtc_array[RTC_MO] % 10) | ((rtc_array[RTC_MO] / 10) << 4));   // month
  rtc_buff[1 + RTC_REG_YR] = (uint8_t)       ((rtc_array[RTC_YR] % 10) | ((rtc_array[RTC_YR] / 10) << 4));    // year

  /* write 9 bytes with time parameters to RTC */
  MSS_I2C_write(&g_mss_i2c0, RTC_ADDR, rtc_buff, RTC_BUFF_SIZE, MSS_I2C_RELEASE_BUS);
  MSS_I2C_wait_complete( &g_mss_i2c0, RTC_TIMEOUT );

}

/**
  * @brief  Function used in FatFs implementation - use external RTC
  * @param  None
  * @retval time word - it is defined according to FatFs implementation
  *         see http://elm-chan.org/fsw/ff/en/fattime.html
  */
DWORD get_fattime(void){
	uint8_t rtc_array[6];
	uint32_t time_word;

  if(rtc_state == RTC_READY){
    get_rtc_time(rtc_array);
    rtc_array[RTC_YR] = (uint8_t) (rtc_array[RTC_YR] + 20);  // RTC starts from year 2000
                                                             // fat function starts from 1980
    time_word = 0;
    time_word = time_word | (uint32_t)  rtc_array[RTC_YR];              // seven bits for year
    time_word = time_word << 4;
    time_word = time_word | (uint32_t) (rtc_array[RTC_MO] & 0x0F);      // four bits for month
    time_word = time_word << 5;
    time_word = time_word | (uint32_t) (rtc_array[RTC_DY] & 0x1F);      // five bits for day of month
    time_word = time_word << 5;
    time_word = time_word | (uint32_t) (rtc_array[RTC_HR] & 0x1F);      // five bits for hour
    time_word = time_word << 6;
    time_word = time_word | (uint32_t) (rtc_array[RTC_MI] & 0x3F);      // six bits for minute
    time_word = time_word << 5;
    time_word = time_word | (uint32_t) ((rtc_array[RTC_SC] & 0x1F)/2);  // four bits for second
  }
  else
    time_word = ((DWORD)(2016 - 1980) << 25 | (DWORD)5 << 21 | (DWORD)1 << 16);
  
	return time_word;


}

/**
  * @brief  Function just return state of RTC initialization
  * @param  None
  * @retval RTC_READY when RTC was sucessfully initialized
  *         RTC_NREADY if not
  */
uint8_t is_rtc_init (void){
  return(rtc_state);


}

/**
  * @brief  Set time of external RTC
  * @param[in]  Time parameter - usese FatFs format - see:
  *             http://elm-chan.org/fsw/ff/en/fattime.html
  * @retval None
  */
void set_fattime(uint32_t time_word){
  uint8_t rtc_array[RTC_SIZE];

  if(rtc_state == RTC_READY){
	// seconds
    rtc_array[RTC_SC] = (uint8_t) ((time_word & 0x1F)*2);
    time_word = time_word >> 5;
    // minute
    rtc_array[RTC_MI] = (uint8_t) (time_word & 0x3F);
    time_word = time_word >> 6;
    // hour
    rtc_array[RTC_HR] = (uint8_t) (time_word & 0x1F);
    time_word = time_word >> 5;
    // day
    rtc_array[RTC_DY] = (uint8_t) (time_word & 0x1F);
    time_word = time_word >> 5;
    // month
    rtc_array[RTC_MO] = (uint8_t) (time_word & 0x0F);
    time_word = time_word >> 4;
    // year
    rtc_array[RTC_YR] = (uint8_t) (time_word & 0x7F);
    if(rtc_array[RTC_YR] <= 20)
      rtc_array[RTC_YR] = 0;
    else
      rtc_array[RTC_YR] = (uint8_t) (((int)rtc_array[RTC_YR]) - 20);

    set_rtc_time(rtc_array);

  }

}

