/**
  ******************************************************************************
  * @file      main.c
  * @author    Marek Laban, Marcel Kleja, Ivan Pecar MICRONIC a.s., LabHC
  * @version   v1.0
  * @note      Created in frame of HECTOR project (No. 644052).
  * @copyright MICRONIC a.s., LabHC
  * @date      16-May-2016
  * @revision  v1.0 created (ML)
  *            v11 (ML)
  *             - added support for fat time
  *             - added support for I2C RTC
  *             - added new functions intended for alarm log:
  *                 write_log(); create_log(); log2file(); GPIO20_IRQHandler();
  *             - added new commmand SET_RTC_TIME
  *             - added initialization for RTC
  *             - KAT test moved to FPGA
  * @brief     This file provides control of HECTOR evaluation motherboard.
  *            Functionalities:
  *             - Configure MSS (Microcontroller SubSystem)
  *             - Operate State machine
  *             - Receive and send packets from/to Host PC
  *             - Operate USB mass storage class device
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "hal/hal.h"
#include "drivers/mss_uart/mss_uart.h"
#include "drivers/mss_gpio/mss_gpio.h"
#include "drivers/mss_i2c/mss_i2c.h"
#include "lpddr/lpddr.h"
#include "ram_drive_app.h"
#include "FatFs/src/ff.h"
#include "drivers_config/sys_config/sys_config.h"
#include "my_uart.h"
#include "main.h"
#include "time.h"
#include "pkt_nfo.h"
#include "fab_nfo.h"
#include "mss_instr.h"
#include "my_vcp.h"
#include "ex_rtc.h"
#include "FatFs/src/ffconf.h"
/* Private define ------------------------------------------------------------*/

/* Data acquisition start address */
#define DATA_AQ_START   RAM_CACHE_ADDR
/* Maximum data acquistion stream size */
#define DATA_AQ_MAX     RAM_CACHE_SIZE

/** @defgroup cmd_state definititions
  * @{
  */
#define NO_CMD    0                      /**< no command received */
#define CMD_RCVD  1                      /**< command received */
#define VCP_OPENED 1
#define VCP_CLOSED 0
#define ONE_BYTE 0xFF                    /**< used to read only last byte */

/*Definitions makes from 32 bit state word 8 bit word */
#define INSTR_WORD(a)  (uint8_t) ( a          & 0xFF) /**< Make instruction word */

#define GPIO_WORD(a)   (uint8_t) ((a >> 8   ) & 0xFF) /**< Make GPIO values word */

/* Initialization definitions for alarm logger */
#define LOG_SIZE 17                     /**< size of buffer which is used for log */
#define LOG_NMB 10                      /**< number of logs in buffer */

#define LOG_OK    0
#define LOG_FAIL  1

/* Timeout in ms for SSI word reading from daughter board */
#define SSI_TIMEOUT 50

/* Private functions ---------------------------------------------------------*/
void    my_system_init(void);
uint8_t acq_check     (void);
void    make_fab_reset(void);
void    make_gen_reset(void);
void    gpio_set      (uint8_t gpio_new_value);
void    gpio_config   (uint8_t gpio_set);
uint8_t gpio_state    (void);
void    write_log     (void);
uint8_t create_log    (void);
uint8_t log2file(void);
extern  void GPIO20_IRQHandler (void);
uint8_t create_dummy_log(void);
void enable_daughter_reset (void);
void disable_daughter_reset(void);

/* Private variables ---------------------------------------------------------*/
static uint8_t  cmd_state = NO_CMD;  /**< command received state */
static uint32_t mss_cmd;             /**< MCU command */
static uint32_t fab_cmd;             /**< fabric command */
static uint32_t stream_size = 0x200; /**< size of transfered stream */
uint8_t op_stage;                    /**< Actual stage of operation in percent */
FIL fil;                             /**< File object */
static uint8_t gpio_mask;            /**< IO Mask type of pins 0 - 7 */
FATFS fs;                            /**< File system object (volume work area) */

uint8_t log_data[LOG_SIZE*LOG_NMB+1];/**< Array used for alarm log + term. char.*/
static uint8_t lg_in = 0;            /**< Number of words inserted in log array */
static uint8_t lg_wr = 0;            /**< Number of written logs */
static uint8_t lg_rd = 0;            /**< Number of read logs */
/**
  * @brief  Initialization and main state machine of mother evaluation board
  * @param  None
  * @retval None
  */
int main(void){
  uint8_t mb_state = MB_IDLE;    /**< Mother Board state */
  uint8_t op_state = OP_IDLE;    /**< Acquisition state */
  uint8_t vcp_state = VCP_CLOSED;
  uint8_t  pclg_nmb;
  uint32_t *pclg_array;
  FIL my_fil;
  UINT br;
  FRESULT res;                   /**< Result of filesystem */
  /* System initialization */
  my_system_init();

  /* make reset of FPGA integrated logic */
  make_fab_reset();
  make_gen_reset();

  /* Stay in State machine cycling */
  while(1){
    /* routine checks written logs, gets time and write them to file */
    while (lg_in > 0){
      log2file();
      lg_in --;           /* decrement of inserted words */
    }
    switch(mb_state){
    /* Idle state of Mother board */
    case MB_IDLE:

      /* When new command received - change state */
      if(cmd_state == CMD_RCVD){
        cmd_state = NO_CMD;
        mb_state = INSTR_WORD(mss_cmd);
        if(mb_state == MB_IDLE)
          send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_NOTHING_TODO);
      }

      /* Check acquisition */
      if(op_state == OP_BUSY){
        op_state = acq_check();
      }

      //if(vcp_state == VCP_OPENED){
      //  send_pp_data();
      //}

      break;

    /* Send FAB Command */
    case MB_FAB_CMD:
      HW_set_32bit_reg(CMD_ADDR,fab_cmd);
      send_resp(fab_cmd,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;

    case MB_ECHO:
      get_pclg(&pclg_array,&pclg_nmb);
      send_pslg_resp(pclg_array,pclg_nmb,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;

    /* Make reset of fabric logic */
    case MAKE_FAB_RST:

      make_fab_reset();
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);

      mb_state = MB_IDLE;
      break;

    /* Make reset of generator */
    case MAKE_GEN_RST:
      make_gen_reset();
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;

    /* Collect generator data  */
    case GEN_DATA_COLLECT:

      /* size of stream must be multiple of 4 */
      stream_size = stream_size + (stream_size % 4);

      /* check size of required stream */
      if(stream_size > DATA_AQ_MAX){
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_AQ_BIG_STREAM);
        mb_state = MB_IDLE;
        break;
      }

      /* check state of data collecting */
      if((op_state != OP_BUSY) && (HW_get_32bit_reg(STATE_ADDR) == OP_IDLE_STATE))
        op_state = OP_BUSY;
      else{
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_OP_INPRG);
        mb_state = MB_IDLE;
        break;
      }

      /* reset stage of acquisition to zero */
      op_stage = 0;

      /* Set start address of stream */
      HW_set_32bit_reg(WSTART_ADDR,DATA_AQ_START);

      /* Set end address of stream */
      HW_set_32bit_reg(WSIZE_ADDR, stream_size);

      /* Start acquisition */
      HW_set_32bit_reg(CMD_ADDR,CMD_OP_ST);

      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);

      mb_state = MB_IDLE;
      break;
    case MB_LOAD_FILE:
      /* size of stream must be multiple of 4 */
      if(stream_size % 4 != 0 )
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_NOT_MULTIPLE);

      /* check size of required stream */
      if(stream_size > DATA_AQ_MAX){
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_AQ_BIG_STREAM);
        mb_state = MB_IDLE;
        break;
      }

      /* check state of FPGA */
      if(HW_get_32bit_reg(STATE_ADDR) == OP_BUSY_STATE){
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_OP_INPRG);
        mb_state = MB_IDLE;
        break;
      }

      /* Mount filesystem */
      f_mount(&fs, "" ,0);

      /* open file */
      if( f_open(&my_fil, (TCHAR *)get_filename(), FA_OPEN_EXISTING | FA_READ) != FR_OK ){
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_FSYS_ERR);
        mb_state = MB_IDLE;
        break;
      }

      /* read data from file */
      if( f_read(&my_fil, (uint8_t*) RAM_CACHE_ADDR,stream_size, &br) != FR_OK){
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_RD_FILE_ERR);
        mb_state = MB_IDLE;
        break;
      }

      /* check number of read bytes */
      if(br != stream_size ){
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_RD_FILE_ERR);
        mb_state = MB_IDLE;
        break;
      }

      /* close file */
      f_close(&my_fil);

      /* umount work area */
      res = f_mount(0, "", 0);

      /* Set start address of stream */
      HW_set_32bit_reg(RSTART_ADDR,DATA_AQ_START);

      /* Set end address of stream */
      HW_set_32bit_reg(RSIZE_ADDR, stream_size);

      /* send response and set idle state */
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;
    /* Mount USB drive */
    case MB_MOUNT_DISK:
      MSS_GPIO_disable_irq(MSS_GPIO_20);   /* during mounting of disk logger is disabled */
      RAM_DRIVE_init();
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;

    /* Get mss operation state */
    case MB_GET_OP_STATE:
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;

    /* Format drive */
    case MB_MAKE_FSYS:
      /* registers file system object to the FatFs module */
      res = f_mount(&fs, "" ,0);
      if(res != FR_OK){
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_FSYS_ERR);
        mb_state = MB_IDLE;
        break;
      }

      /* make filesystem */
      res = f_mkfs("", 0, 1);
      if(res != FR_OK){
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_FSYS_ERR);
        mb_state = MB_IDLE;
        break;
      }
      f_setlabel("0:HECTOR");

      /* Unregister work area */
      res = f_mount(0, "", 0);
      if(res != FR_OK){
        send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_FSYS_ERR);
        mb_state = MB_IDLE;
        break;
      }

      if (is_rtc_init() == RTC_READY){
        /* During formating of disk, log data are also formated */
        lg_in = 0;
        lg_wr = 0;
        lg_rd = 0;

        create_log();
        MSS_GPIO_enable_irq(MSS_GPIO_20);
      }
      else{
        create_dummy_log();


      }

      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;

    case MB_OP_RST:
      /* make fabric reset */
      make_fab_reset();

      /* reset operation state progress to zero */
      op_stage = 0;
      op_state = OP_IDLE;
      mb_state = MB_IDLE;
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      break;
    case MB_GPIO_CGF:

      gpio_config(GPIO_WORD(mss_cmd));

      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;

    case MB_GPIO_SET:

      gpio_set(GPIO_WORD(mss_cmd));

      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;
    case SEND_FAB_RSP:
      send_fab_resp();
      mb_state = MB_IDLE;
      break;
    case SEND_CRYP_RSP:
      /* Wait while Check delay (cryp_wait) signal */
      while((HW_get_32bit_reg(MSSIO_IN_ADDR) & 0x01) == 1){};
      //send_cryp_resp();
      mb_state = MB_IDLE;
      break;

    case MB_MOUNT_VCP:
      //vcp_state = VCP_OPENED;
      //VCP_conf();
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;
    case SEND_VCP_DATA:
      //if(vcp_state == VCP_OPENED){
    	  //inc_ppdata2vcp_cnt(fab_cmd);
      //}
      send_pp_data();
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;
    case SET_RTC_TIME:
      set_fattime(fab_cmd);
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;
    case INIT_I2C_RTC:
      /* Init I2C bus for RTC */
      init_rtc();

      /* Configure GPIO and IRQ for TRNG alarm */
      MSS_GPIO_config(MSS_GPIO_20, MSS_GPIO_INPUT_MODE|MSS_GPIO_IRQ_EDGE_POSITIVE);
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;
    case TEST_CALL:
      //conf_test();
      write_log();
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_SUCCESS);
      mb_state = MB_IDLE;
      break;
    default:
      send_resp(MB_NOTHING,op_stage, op_state, gpio_state(), MB_CMD_NFND);
      mb_state = MB_IDLE;
      break;
    }
  }
}

/**
  * @brief  Initialization of evaluation board system
  * @param  None
  * @retval None
  */
void my_system_init(void){

  /* set systick to every 1 ms */
  SysTick_Config(MSS_SYS_M3_CLK_FREQ/1000);
  NVIC_EnableIRQ(SysTick_IRQn);

  MSS_GPIO_init();

    /* configure reset of fabric logic */
  MSS_GPIO_config(MSS_GPIO_8, MSS_GPIO_OUTPUT_MODE);
  MSS_GPIO_set_output(MSS_GPIO_8, 0);  // enable reset

  /* configure reset of fabric generator logic */
  MSS_GPIO_config(MSS_GPIO_9, MSS_GPIO_OUTPUT_MODE);
  MSS_GPIO_set_output(MSS_GPIO_9, 0);   // enable reset

  /* configure reset of USB physical layer */
  MSS_GPIO_config(MSS_GPIO_29, MSS_GPIO_OUTPUT_MODE);
  MSS_GPIO_set_output(MSS_GPIO_29, 1);   // enable reset

  /* set all pins 0 - 7 as inputs*/
  gpio_config(0x00);

  /* external RAM configuration */
  MEM_init();

   /* UART configuration */
  my_uart_cfg();
}


/**
  * @brief  Set new MSS state machine command
  * @param[in] new_cmd  new MSS command
  * @retval None
  */
void set_mss_cmd(uint32_t new_cmd){
  cmd_state = CMD_RCVD;
  mss_cmd = new_cmd;
}

/**
  * @brief  Set new word for FABRIC
  * @param[in] new_word  new FABRIC word
  * @retval None
  */
void set_fab_cmd(uint32_t new_word){
  fab_cmd = new_word;
}

/**
  * @brief  Make reset of control fabric logic
  * @param  None
  * @retval None
  */
void make_fab_reset(void){
  uint32_t tmp_word;     /*< Temporary word     */
 
  tmp_word = HW_get_32bit_reg(MSSIO_OUT_ADDR);
  tmp_word = tmp_word | 0x00000200;

  HW_set_32bit_reg(MSSIO_OUT_ADDR,tmp_word);
  Delay(100);

  tmp_word = tmp_word & 0xFFFFFDFF;
  HW_set_32bit_reg(MSSIO_OUT_ADDR,tmp_word);
}

/**
  * @brief  Make generator reset
  * @param  None
  * @retval None
  */
void make_gen_reset(void){
  uint32_t tmp_word;     /*< Temporary word     */
 
  tmp_word = HW_get_32bit_reg(MSSIO_OUT_ADDR);
  tmp_word = tmp_word | 0x00000100;

  HW_set_32bit_reg(MSSIO_OUT_ADDR,tmp_word);
  Delay(100);

  tmp_word = tmp_word & 0xFFFFFEFF;
  HW_set_32bit_reg(MSSIO_OUT_ADDR,tmp_word);

}

/**
  * @brief  Set size of acquisition stream size
  * @param  None
  * @retval None
  */
void set_stream_size(uint32_t stream_word){
  stream_size = stream_word;
}

/**
  * @brief  Check operation state and if it is successful finished,
  *         copy file from cache to file system
  * @param  None
  * @retval New acquisition state
  */
uint8_t acq_check(void){
  FRESULT res;       /**< Result of filesystem */
  uint32_t gen_state;  /**< Generator state */
  uint32_t op_stage_i;/**< Operation stage auxiliary */
  UINT bw;           /**< Bytes written */

  /* read state of generator and return state */
  gen_state = HW_get_32bit_reg(STATE_ADDR);
  switch(gen_state){

  /* When acquisition is in progress, calculate actual state of data collecting */
  case OP_BUSY_STATE:
    op_stage_i = HW_get_32bit_reg(STAGE_ADDR);
    op_stage = (uint8_t)(((float)(op_stage_i - DATA_AQ_START) * (float) 1.0 / ((float)stream_size))*100.0);
    return(OP_BUSY);

  /* When buffer overflow */
  case OP_OFLW_STATE:
    return(OP_OVFLW);

  /* When acquisition is finished */
  case OP_IDLE_STATE:
    /*set stage to end of the operation */
    op_stage = 100;

    res = f_mount(&fs, "" ,0);
    if(res != FR_OK){
      return(OP_WR_ERR);
    }

    res = f_open(&fil, get_filename(), FA_CREATE_ALWAYS | FA_WRITE);
    if(res != FR_OK){
      f_close(&fil);
      return(OP_WR_ERR);
    }

    /* Copy cache to a file */
    res = f_write(&fil, (uint32_t *)(DATA_AQ_START),stream_size, &bw);
    if(res != FR_OK || bw != stream_size){
      f_close(&fil);
      return(OP_WR_ERR);
    }

    /* Close the file */
    f_close(&fil);

    /* Unregister work area */
    res = f_mount(0, "", 0);
    if(res != FR_OK){
      return(OP_WR_ERR);
    }
    return(OP_IDLE);

  default: return(OP_UNKNW);
  }


}


/**
  * @brief  Set values to 8 GPIOs  number 0 - 7.
  * @param  gpio_new_value    New value for gpio pins, where MSB bit means 7. pin
  *                           and LSB bit means 0. pin
  * @retval None
  */
void gpio_set(uint8_t gpio_new_value){
  uint32_t gpio_values;

  /* Set new 8bits value of 32 bit word */
  gpio_values = ((uint32_t) gpio_new_value) & 0xFF;

  /* Get new 32 bit of actual gpio pins and set new value to word*/
  gpio_values = (HW_get_32bit_reg(MSSIO_OUT_ADDR) & 0xFFFFFF00) | gpio_values;

  /* Set new 8 bit word to 0 - 7 pins */
  HW_set_32bit_reg(MSSIO_OUT_ADDR,gpio_values);
}

/**
  * @brief  Return values of used GPIOs (8 - 15.)
  * @param  gpio_new_value    New value for gpio pins, where MSB bit means 15. pin
  *                           and LSB bit means 8. pin
  * @retval 8 bit of GPIOs value, where MSB bit is pin 15. and LSB bit is bit 8.
  */
uint8_t gpio_state(void){
  uint8_t gpio_in;
  uint8_t gpio_out;

  /* read gpio input value */
  gpio_in  = (uint8_t) (((HW_get_32bit_reg(MSSIO_IN_ADDR)) & (uint32_t)~gpio_mask) & 0xFF);

  /* read gpio output value */
  gpio_out = (uint8_t) ((HW_get_32bit_reg(MSSIO_OUT_ADDR)) & (uint32_t) gpio_mask);

  /* read 7. - 0. gpio pins */
  return(gpio_in | gpio_out);
}

/**
  * @brief  Configure used GPIOs (0 - 7.)
  * @param  gpio_set    New configuration for gpio pin, where MSB bit means 15. pin
  *                     and LSB bit means 8. pin.
  *                       - GPIO_CFG_OUT (non-zero bit) means output
  *                       - GPIO_CFG_IN  (zero bit) means input
  * @retval None
  */
void gpio_config(uint8_t gpio_set){
  /* remember configured mask of gpio */
  gpio_mask = gpio_set;

}

/**
  * @brief  Create log file, where log data will be written during measurement
  * @param  None
  * @retval LOG_FAIL when data was written unsucessfull
  *         LOG_OK   when data was written sucessfull
  */
uint8_t create_log(void){
  FRESULT res;         /**< Result of filesystem */
  uint8_t rtc_data[6]; /**< RTC data array */
  UINT bw;             /**< Bytes written */
  char time_data[62];  /**< Array used for written word */

  /* registers file system object to the FatFs module */
  res = f_mount(&fs, "" ,0);
  if(res != FR_OK){
    f_mount(0, "", 0);
    return LOG_FAIL;
  }
  /* Get RTC time */
  get_rtc_time(rtc_data);
  /* Insert time data to string */
  sprintf(time_data,"#Created %02d%02d%02d-%02d:%02d:%02d\r\n DATE    TIME    SSI_HIGH SSI_LOW\r\n",
                              rtc_data[RTC_YR],
                              rtc_data[RTC_MO],
                              rtc_data[RTC_DY],
                              rtc_data[RTC_HR],
                              rtc_data[RTC_MI],
                              rtc_data[RTC_SC]);

  /* Create log file */
  res = f_open(&fil, "alarm.log", FA_CREATE_ALWAYS | FA_WRITE);
  if(res != FR_OK){
    f_close(&fil);
    f_mount(0, "", 0);
    return(LOG_FAIL);
  }

  /* Copy cache to a file */
  res = f_write(&fil, time_data, 61, &bw);
  if(res != FR_OK || bw != 61){
    f_close(&fil);
    f_mount(0, "", 0);
    return(LOG_FAIL);
  }

  /* Close the file */
  f_close(&fil);

  /* Unregister work area */
  res = f_mount(0, "", 0);
  if(res != FR_OK){
    return(LOG_FAIL);
  }

  return(LOG_OK);

}

/**
  * @brief  Create log file, where log data will be written during measurement
  * @param  None
  * @retval LOG_FAIL when data was written unsucessfull
  *         LOG_OK   when data was written sucessfull
  */
uint8_t create_dummy_log(void){
  FRESULT res;         /**< Result of filesystem */
  UINT bw;             /**< Bytes written */
  char time_data[27];  /**< Array used for written word */

  /* registers file system object to the FatFs module */
  res = f_mount(&fs, "" ,0);
  if(res != FR_OK){
    f_mount(0, "", 0);
    return LOG_FAIL;
  }
  /* Insert time data to string */
  sprintf(time_data,"RTC module not detected.\r\n");

  /* Create log file */
  res = f_open(&fil, "alarm.log", FA_CREATE_ALWAYS | FA_WRITE);
  if(res != FR_OK){
    f_close(&fil);
    f_mount(0, "", 0);
    return(LOG_FAIL);
  }

  /* Copy cache to a file */
  res = f_write(&fil, time_data, 27, &bw);
  if(res != FR_OK || bw != 61){
    f_close(&fil);
    f_mount(0, "", 0);
    return(LOG_FAIL);
  }

  /* Close the file */
  f_close(&fil);

  /* Unregister work area */
  res = f_mount(0, "", 0);
  if(res != FR_OK){
    return(LOG_FAIL);
  }

  return(LOG_OK);

}


/**
  * @brief  Take 2 state words from fabric and insert them to array
  * @param  None
  * @retval None
  */
void write_log(void){
  uint32_t ssi_high;
  uint32_t ssi_low;
  ela_t my_ela;
  uint8_t elapsed = 0;

  /*Configure SSI registers and start SSI(insert zero to them) */
  HW_set_32bit_reg(FABIN_ADDR,(uint32_t)0x00);
  HW_set_32bit_reg(CMD_ADDR,CMD_SSI_LOAD_HIGH);
  HW_set_32bit_reg(CMD_ADDR,CMD_SSI_LOAD_SEND_LOW);

  /* Set timer and wait while SSI finished data reading */
  set_elapsed(my_ela,SSI_TIMEOUT);
  while((HW_get_32bit_reg(FABST_ADDR) & 0x00000001) == 0){
	  if(t_elapsed(my_ela) == TIM_ELA){
		elapsed = 1;
        break;
	  }
  };

  /* If reading was sucessfull read SSI registers */
  if(elapsed == 0){

	HW_set_32bit_reg(CMD_ADDR,CMD_READ_SSI_HIGH);
    ssi_high = HW_get_32bit_reg(FABOUT_ADDR);
    HW_set_32bit_reg(CMD_ADDR,CMD_READ_SSI_LOW);
    ssi_low  = HW_get_32bit_reg(FABOUT_ADDR);

    /* make string from words */
    sprintf(&log_data[lg_wr],"%08X %08X",ssi_high, ssi_low);

  }
  else{
	/* Insert error message to log */
    sprintf(&log_data[lg_wr],"SSI timer elapsed");
  }

  /* Increment number of logs in array */
  lg_in ++;
  lg_wr = lg_wr + LOG_SIZE;

  /* Make circle buffer */
  if (lg_wr >= (LOG_SIZE*LOG_NMB))
    lg_wr = 0;
}

/**
  * @brief  Insert log data to file
  * @param  None
  * @retval LOG_FAIL when data was written unsucessfull
  *         LOG_OK   when data was written sucessfull
  */
uint8_t log2file(void){
  FRESULT res;        /**< Result of filesystem */
  UINT bw;            /**< Bytes written */
  uint8_t rtc_data[6];
  char time_data[17]; /**< Log word which is inserted to file */

  /* Mount disk */
  res = f_mount(&fs, "" ,0);
  if(res != FR_OK){
    return(LOG_FAIL);
  }

  /* Open log file */
  res = f_open(&fil, "alarm.log", FA_WRITE | FA_OPEN_EXISTING);
  if(res != FR_OK){
    f_close(&fil);
    f_mount(0, "", 0);
    return(LOG_FAIL);
  }

  /* Go to end of file */
  res = f_lseek(&fil, f_size(&fil));
  if(res != FR_OK){
    f_close(&fil);
    f_mount(0, "", 0);
    return(LOG_FAIL);
  }

  /* Get time and insert it to string */
  get_rtc_time(rtc_data);
  sprintf(time_data,"%02d%02d%02d-%02d:%02d:%02d ",
                     rtc_data[RTC_YR],
                     rtc_data[RTC_MO],
                     rtc_data[RTC_DY],
                     rtc_data[RTC_HR],
                     rtc_data[RTC_MI],
                     rtc_data[RTC_SC]);

  /* Copy time to the file */
  res = f_write(&fil, time_data, 17, &bw);
  if(res != FR_OK || bw != 17){
    f_close(&fil);
    f_mount(0, "", 0);
    return(LOG_FAIL);
  }

  /* Copy ssi words to the file */
  res = f_write(&fil, &log_data[lg_rd], LOG_SIZE, &bw);
  if(res != FR_OK || bw != LOG_SIZE){
    f_close(&fil);
    f_mount(0, "", 0);
    return(LOG_FAIL);
  }

  /* Copy cache to a file */
  res = f_write(&fil, "\r\n", 2, &bw);
  if(res != FR_OK || bw != 2){
    f_close(&fil);
    f_mount(0, "", 0);
    return(LOG_FAIL);
  }

  /* Increment read logs pointer */
  lg_rd = lg_rd + LOG_SIZE;
  if (lg_rd >= (LOG_SIZE*LOG_NMB))
    lg_rd = 0;

  /* Close the file */
  f_close(&fil);

  /* Unregister work area */
  res = f_mount(0, "", 0);
  if(res != FR_OK){
    return(LOG_FAIL);
  }

  return(LOG_OK);


}

void enable_daughter_reset(void){
  uint32_t tmp_word;     /*< Temporary word     */

  tmp_word = HW_get_32bit_reg(MSSIO_OUT_ADDR);
  tmp_word = tmp_word | 0x00000100;

  HW_set_32bit_reg(MSSIO_OUT_ADDR,tmp_word);

}

void disable_daughter_reset(void){
  uint32_t tmp_word;     /*< Temporary word     */

  tmp_word = HW_get_32bit_reg(MSSIO_OUT_ADDR);

  tmp_word = tmp_word & 0xFFFFFEFF;
  HW_set_32bit_reg(MSSIO_OUT_ADDR,tmp_word);


}


/**
  * @brief  Inerrupt routine - Occurs on rising edge of GPIO 20
  *         Immediately write ssi word to array
  * @param  None
  * @retval None
  */
void GPIO20_IRQHandler( void ){

  if (lg_in < LOG_NMB)                 /* When buffer is not full*/
    write_log();                       /* Write ssi word to log */
  MSS_GPIO_clear_irq (MSS_GPIO_20   ); /* Clear interrupt */
}
