-------------------------------------------------------------------------------
--
-- Design unit:   Control block
--                 
--
-- File name:     ctrl.vhd
--
-- Description:   Control block is a fundamental block in the  HECTOR evaluation 
--                platform, it consists of two components: ahb_reg and ram_if. 
--                Ahb_reg implements registers which are available from MSS. 
--                Ram_if ensure data transfer from/to the external RAM. The control 
--                block uses these two components to operate with user entity 
--                and transfer data to the host PC. A lot of signals and registers 
--                are not used, they are used only as a template for making a
--                user adjustable system. 
--
--                This user adjustable system consists of two tools, the first 
--                one is a command execution state machine (CMD_EXE) and the 
--                second one is a user application logic. 
--
--                CMD_EXE executes command when new data are written to the CMD
--                register. Commands can be triggered by Fab packet (packet 
--                writes data to command and fabin register). New command can be 
--                added by creating of new command definition (constant) and its 
--                inserting to CMD_EXE.
--
--                Example is designed to acquire ouput of TRNG in external 
--                daughter board. It collects 32-bit words from par2par 
--                converter and save it to the external RAM memory.
-- 
--
-- System:        VHDL'93
--
-- Author:        Marek Laban, Marcel Kleja, Ivan Pecar MICRONIC a.s.,  
--                Oto Petura, LabHC
--
-- Copyright:     MICRONIC a.s., LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 1.00, May 2016
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
-------------------------------------------------------------------------------
LIBRARY ieee;

USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_arith.ALL;
USE ieee.std_logic_unsigned.ALL;

LIBRARY work;
USE work.mb_pkg.ALL;


ENTITY ctrl IS
  PORT (
  -----------------------------------------------------------------------------
  -- AHB MASTER bus (external RAM) --------------------------------------------
  -----------------------------------------------------------------------------
    htrans     : OUT STD_LOGIC_VECTOR(1  DOWNTO 0);   
    hsize      : OUT STD_LOGIC_VECTOR(1  DOWNTO 0);
    hwrite     : OUT STD_LOGIC;    
    hwdata     : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);   
    haddr      : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    hmastlock  : OUT STD_LOGIC;            
    hsel       : OUT STD_LOGIC;  
    hreadyout  : IN  STD_LOGIC; 
    hresp      : IN  STD_LOGIC;       
    hrdata     : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hburst     : OUT STD_LOGIC_VECTOR(2  DOWNTO 0);
    --
  -----------------------------------------------------------------------------
  -- AHB SLAVE bus (MSS)-------------------------------------------------------
  -----------------------------------------------------------------------------
    haddr_mss  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hwdata_mss : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hrdata_mss : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    hsel_mss   : IN  STD_LOGIC;   
    hready_mss : OUT STD_LOGIC;   
    hwrite_mss : IN  STD_LOGIC;
    hrdy_mss   : IN  STD_LOGIC;
    --
  -----------------------------------------------------------------------------
  -- Control block signals and registers --------------------------------------
  -----------------------------------------------------------------------------
  -- global signals 
    rst        : IN  STD_LOGIC;
    clk        : IN  STD_LOGIC;
  -- S/P signals
    sp_data    : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- data from SP
    sp_data_vld: IN  STD_LOGIC;                      -- data valid from SP
  -- HDMI TRNG status
    hdmi_ssi_tx    : OUT STD_LOGIC;
    hdmi_ssi_rx    : IN  STD_LOGIC;
    hdmi_ssi_strb  : IN  STD_LOGIC;
    hdmi_rng_alarm : IN  STD_LOGIC;
  -- SATA TRNG status
    sata_ssi_tx    : OUT STD_LOGIC;
    sata_ssi_rx    : IN  STD_LOGIC;
    sata_ssi_strb  : IN  STD_LOGIC;
    sata_rng_alarm : IN  STD_LOGIC;
    rng_alarm_irq  : OUT STD_LOGIC;  -- rng alarm to MSS interrupt request
  -- 
  -- multiplexor out (0 - HDMI,1 - SATA)
    db_mux_out : OUT STD_LOGIC;
  -- daughter board reset signal  
    db_reset   : OUT STD_LOGIC;
  -- select output of the daughter board 
    mux_sel    : OUT STD_LOGIC;
 
  -- Output to a LED              
    led_out      : OUT STD_LOGIC;
  --
  -- 'Sata' daughter board power supply enable (DEMO1)
    sata_pwr_en  : OUT STD_LOGIC;
  -- 'Hdmi' daughter board power supply enable (DEMO1)
    hdmi_pwr_en  : OUT STD_LOGIC;
  --
  -- state signal of selected USB connector (DEMO1)
    usb_mux      : IN  STD_LOGIC;

    d1_par_err   : IN STD_LOGIC;
  --
    d2_par_err   : IN STD_LOGIC
  );
END ctrl;

ARCHITECTURE arch_ctrl OF ctrl IS

  -- BEGIN COMPONENT DEFINITIONS **********************************************

  COMPONENT ram_if IS
  PORT(
  -- global reset and clock signals
    rst              : IN    STD_LOGIC;                  
    clk              : IN    STD_LOGIC;                  
  ---------------------------------------------------------------------------- 
  -- AHB bus signals ---------------------------------------------------------
  ----------------------------------------------------------------------------
    -- AHB-Lite transfer type for master. 
    -- Indicates the type of the current transfer
    htrans            : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);   
    -- AHB-Lite transfer size for master. Indicates the size of the transfer. 
    -- Only byte, half-word and word transactions are supported
    hsize             : OUT   STD_LOGIC_VECTOR(1 DOWNTO 0);
    -- AHB-Lite write indication for master 0. 
    -- High for a write, Low for a read.           
    hwrite            : OUT   STD_LOGIC;    
    -- AHB-Lite write data for master                   
    hwdata            : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);   
    -- AHB-Lite address
    haddr             : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- AHB-Lite locked sequence indication for master 0      
    hmastlock         : OUT   STD_LOGIC;            
    -- AHB-Lite slave 0 select                     
    hsel              : OUT   STD_LOGIC;  
    -- AHB-Lite ready signal from slave 0                    
    hreadyout         : IN    STD_LOGIC; 
    -- AHB-Lite transfer response for master                     
    hresp             : IN    STD_LOGIC;       
    -- AHB-Lite read data for master              
    hrdata            : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- AHB-Lite burst register output
    hburst            : OUT   STD_LOGIC_VECTOR(2 DOWNTO 0);
    --
  ----------------------------------------------------------------------------
  -- Interface for R/W data to/from external RAM memmory ---------------------
  ----------------------------------------------------------------------------
    -- data input and valid input data signal
    data_in           : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    data_in_vld       : IN    STD_LOGIC;  -- data are valid during asserting 1 
    -- data output and valid output data signal
    data_out          : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0);
    data_out_vld      : OUT   STD_LOGIC;  -- data are valid during asserting 1
    --
    -- start address of DMA 
    addr_in           : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- bytes count (number of 8-bit words (rounded to multiple of 4))
    byte_cnt          : IN    STD_LOGIC_VECTOR(31 DOWNTO 0);
    --
    -- control signals  
    data_wr           : IN    STD_LOGIC;  -- data read(0) or data write(1) 
    ram_strt          : IN    STD_LOGIC;  -- ram start (1 during one clk period)
    -- info signals  
    wr_buf_fl         : OUT   STD_LOGIC;  -- write buffer full (1 when full)
    wr_buf_half       : OUT   STD_LOGIC;  -- write buffer is half full
    buff_ofw          : OUT   STD_LOGIC;  -- buffer overflow (1 when overflow)
    ram_bsy           : OUT   STD_LOGIC;  -- ram busy (1 when busy, 0 when idle) 
    -- actual address of ram opertion
    ram_stg           : OUT   STD_LOGIC_VECTOR(31 DOWNTO 0) 
  );
  END COMPONENT;
  
  COMPONENT ahb_reg IS
  PORT (
  -- global signals 
    rst        : IN    STD_LOGIC;                  
    clk        : IN    STD_LOGIC;
    --
  -- AHB SLAVE bus (MSS)
    haddr_mss  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hwdata_mss : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    hrdata_mss : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    hsel_mss   : IN  STD_LOGIC;   
    hready_mss : OUT STD_LOGIC;   
    hwrite_mss : IN  STD_LOGIC;
    hrdy_mss   : IN  STD_LOGIC;
    -- 
  -- registers available from MSS
    cmd_reg    : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- command register
    state_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- state of operation
    stage_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- actual used address
    --
    fabin_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric input (from MSS)
    fabout_reg : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric output (to MSS)
    fabst_reg  : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);  -- fabric status (to MSS)
    --
    -- parameters where will be saved data for user file generation 
    -- (RAM to MSS file)
    wstart_reg : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- start address
    wsize_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- size in bytes
    --
    -- parameters where will be loaded user file (MSS file to RAM)
    rstart_reg : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- start address
    rsize_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  -- size in bytes
    -- registers used for GPIO input/output from MSS
    mssio_out_reg  : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);  
    mssio_in_reg   : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
    --
    pp_read        : OUT STD_LOGIC;                    -- pp data read request
    pp_drdy        : IN STD_LOGIC;                     -- pp data ready
    pp_data        : IN STD_LOGIC_VECTOR (31 DOWNTO 0);-- pp data input      
  -- new command received signal (assert high)
    new_cmd        : OUT STD_LOGIC
  -----------------------------------------------------------------------------
  );
  END COMPONENT;

-- DB SSI
  COMPONENT ssi IS
    PORT (
      reset    : IN  STD_LOGIC;     -- Positive reset
      clk      : IN  STD_LOGIC;     -- Input clock
      --
      -- data input/output from device
      ssi_rx   : IN  STD_LOGIC;     -- RX serial line
      ssi_tx   : OUT STD_LOGIC;     -- TX serial line
      ssi_strb : IN  STD_LOGIC;     -- ssi strobe signal 
      --
      -- tx side ------------------------------------------  
      -- Parallel data input
      tx_data  : IN  STD_LOGIC_VECTOR ( 63 DOWNTO 0 ); 
      tx_iprg  : OUT STD_LOGIC;     -- transfer in progress
      tx_start : IN  STD_LOGIC;     -- when asserted, data are transmitted by ssi
      --
      -- rx side ------------------------------------------
      -- Parallel data output
      rx_data  : OUT STD_LOGIC_VECTOR ( 63 DOWNTO 0 );
      data_rdy : OUT STD_LOGIC      -- RX data ready
    );
  END COMPONENT ssi;

  -- END COMPONENT DEFINITIONS ************************************************


  -- BEGIN COMMAND ************************************************************
  -- command execution signals and registers 
    -- command state machine states
    TYPE cmd_state_type IS( CMD_IDLE,
                            CMD_WAIT_THEN_IDLE,
                            CMD_EXE
                           ); 
    SIGNAL cmd_state        : cmd_state_type; 
    -- new command was received signal
    SIGNAL new_cmd          : STD_LOGIC;       
    -- command register
    SIGNAL cmd_reg          : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- fabric input register (from MSS)
    SIGNAL fabin_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- fabric output register (to MSS)
    SIGNAL fabout_reg       : STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- fabric status register (to MSS)
    SIGNAL fabst_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);
    --
    -- multiplexing between daughter boards (0 - HDMI, 1 - SATA connector)
    SIGNAL mux_out          : STD_LOGIC;      
  -- END COMMAND ***************************************************************
  


  -- BEGIN TEMPLATE DESIGN SIGNALS ********************************************* 
  -- state of acquistion register definitions  
    -- acquisition idle state
    CONSTANT OP_IDLE_STATE  : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"FFFFFFFF"; 
    -- busy state
    CONSTANT OP_BUSY_STATE  : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000001"; 
    -- buffer overflow state
    CONSTANT OP_OFLW_STATE  : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000002"; 
    -- buffer overflow state
    CONSTANT OP_ERR_STATE   : STD_LOGIC_VECTOR(31 DOWNTO 0) := x"00000003"; 
    --
    -- start address of write ram interface operation
    SIGNAL wstart_reg       : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- size in bytes of write ram interface operation
    SIGNAL wsize_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- start address of read ram interface operation
    SIGNAL rstart_reg       : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- size in bytes of read ram interface operation
    SIGNAL rsize_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    --
    -- user optional register
    SIGNAL mssio_out_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    -- user optional register    
    SIGNAL mssio_in_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    --    
    -- operation state register
    SIGNAL state_reg        : STD_LOGIC_VECTOR(31 DOWNTO 0);  
    --
    -- operation state machine states
    TYPE op_state_type IS     ( OP_IDLE,
                                OP_READ_DATA,
                                OP_OFLW,
                                OP_PP_READ
                              ); 
    SIGNAL op_state     : op_state_type;
    --
    -- operation enable signal
    SIGNAL op_enable    : STD_LOGIC;   
    -- data input of ram interface
    SIGNAL ram_in       : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    -- valid data input of ram interface
    SIGNAL ram_in_vld   : STD_LOGIC;    
    --
    -- data output of ram interface
    SIGNAL ram_out      : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    -- valid data ouput of ram interface
    SIGNAL ram_out_vld  : STD_LOGIC;    
    --
    -- start address of ram interface operation
    SIGNAL ram_addr     : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    -- number of bytes of ram interface operation           
    SIGNAL ram_cnt      : STD_LOGIC_VECTOR(31 DOWNTO 0); 
    --
    -- ram interface type of operation (1 - write, 0 - read)
    SIGNAL ram_rw       : STD_LOGIC;    
    -- ram interface start signal (assert 1 during one clk period)
    SIGNAL ram_strt     : STD_LOGIC;
    --
    -- read buffer not empty  
    SIGNAL rd_buf_ne    : STD_LOGIC;
    -- write buffer full  
    SIGNAL wr_buf_fl    : STD_LOGIC;
    -- write buffer full  
    SIGNAL wr_buf_half  : STD_LOGIC;
    -- write buffer overflow occurs  
    SIGNAL ram_ofw      : STD_LOGIC;
    -- 
    -- actual used address of ram interface
    SIGNAL stage_reg    : STD_LOGIC_VECTOR(31 DOWNTO 0);
    -- ram interface busy (1 - r/w operation occurs) 
    SIGNAL ram_bsy      : STD_LOGIC;
 

   -- END TEMPLATE DESIGN SIGNALS ***********************************************

    -- SP converter signals
    SIGNAL sp_data_vld_i: STD_LOGIC;  -- auxiliary data valid signal
    SIGNAL vld_sample   : STD_LOGIC;  -- sample signal of SP converter

    -- DB SSI connection
    SIGNAL tx_start : STD_LOGIC;
    SIGNAL tx_iprg  : STD_LOGIC;
    SIGNAL tx_data  : STD_LOGIC_VECTOR(63 DOWNTO 0);
    SIGNAL rx_data  : STD_LOGIC_VECTOR(63 DOWNTO 0);
    SIGNAL data_rdy : STD_LOGIC;

    SIGNAL hdmi_tx_start : STD_LOGIC;
    SIGNAL hdmi_tx_iprg  : STD_LOGIC;
    SIGNAL hdmi_tx_data  : STD_LOGIC_VECTOR(63 DOWNTO 0);
    SIGNAL hdmi_rx_data  : STD_LOGIC_VECTOR(63 DOWNTO 0);
    SIGNAL hdmi_data_rdy : STD_LOGIC;    
    
    SIGNAL sata_tx_start : STD_LOGIC;
    SIGNAL sata_tx_iprg  : STD_LOGIC;
    SIGNAL sata_tx_data  : STD_LOGIC_VECTOR(63 DOWNTO 0);
    SIGNAL sata_rx_data  : STD_LOGIC_VECTOR(63 DOWNTO 0);
    SIGNAL sata_data_rdy : STD_LOGIC;        

    SIGNAL ssi_data_latch   : STD_LOGIC_VECTOR(63 DOWNTO 0);
    SIGNAL ssi_data_start_s : STD_LOGIC;
    SIGNAL ssi_data_ready_s : STD_LOGIC;
    SIGNAL ssi_in_prog_s    : STD_LOGIC;
    
    TYPE   ssi_states IS (IDLE, WAIT_FOR_DATA);
    SIGNAL ssi_state     : ssi_states := IDLE;

    SIGNAL ms_cnt   : STD_LOGIC_VECTOR(31 DOWNTO 0) := (OTHERS => '0');
    SIGNAL time_cnt : STD_LOGIC_VECTOR(31 DOWNTO 0) := (OTHERS => '0');

    TYPE data_requests IS ( REQ_NONE,
                            REQ_RNG_STATUS,
                            REQ_TIME,
                            REQ_SSI_LOW_32,
                            REQ_SSI_HIGH_32
                          );
    SIGNAL requested_data : data_requests := REQ_NONE;

    SIGNAL db_rst : STD_LOGIC := '0'; -- Daughter board positive reset
    SIGNAL fab_rst: STD_LOGIC;        -- Fabric logic reset
    -- total failure alarm
    SIGNAL rng_alarm : STD_LOGIC;
BEGIN

  i_ram_if: ram_if 
  PORT MAP (
  -- global signals  
    rst        => fab_rst,
    clk        => clk,

  -- AHB signals and registers (every is led out to upper entity) --
    htrans     => htrans, 
    hsize      => hsize, 
    hwrite     => hwrite,
    hwdata     => hwdata,
    haddr      => haddr,
    hmastlock  => hmastlock,
    hsel       => hsel,
    hreadyout  => hreadyout,
    hresp      => hresp,
    hrdata     => hrdata,
    hburst     => hburst,
    --
  -- user interface signals --
    data_in      => ram_in,
    data_in_vld  => ram_in_vld,
    --
    data_out     => ram_out,
    data_out_vld => ram_out_vld,
    --
    addr_in    => ram_addr,
    byte_cnt   => ram_cnt,
    --
    data_wr    => ram_rw,
    ram_strt   => ram_strt,
    --
    wr_buf_fl  => wr_buf_fl,
    wr_buf_half=> wr_buf_half,
    buff_ofw   => ram_ofw,
    --
    ram_stg    => stage_reg,
    ram_bsy    => ram_bsy
  );

  i_ahb_reg: ahb_reg
  PORT MAP (
  -- global signals  
    rst        => rst,
    clk        => clk,
    --
  -- AHB SLAVE bus (MSS) 
    haddr_mss  => haddr_mss,
    hwdata_mss => hwdata_mss,
    hrdata_mss => hrdata_mss,
    hsel_mss   => hsel_mss,   
    hready_mss => hready_mss,   
    hwrite_mss => hwrite_mss,
    hrdy_mss   => hrdy_mss,
    --
  -- registers available from MSS
    cmd_reg    => cmd_reg,
    state_reg  => state_reg,
    stage_reg  => stage_reg,
    -- registers available from user instructions (UART)
    fabin_reg  => fabin_reg, 
    fabout_reg => fabout_reg,
    fabst_reg  => fabst_reg,
    --
    wstart_reg => wstart_reg,
    wsize_reg  => wsize_reg,
    rstart_reg => rstart_reg,
    rsize_reg  => rsize_reg,
    -- user optional registers
    mssio_out_reg => mssio_out_reg,
    mssio_in_reg  => mssio_in_reg,
    -- reset of the daughter board
    --
	pp_read      => OPEN,
    pp_drdy      => '0',
    pp_data      => x"00000000",
	--
  -- new command received signal (assert high)
    new_cmd    => new_cmd
  );

  ssi_hdmi_inst : ssi
    PORT MAP (
      reset    => db_rst,
      clk      => clk,
      
      -- slave device interface
      ssi_rx   => hdmi_ssi_rx,
      ssi_tx   => hdmi_ssi_tx,
      ssi_strb => hdmi_ssi_strb,
      
      -- tx side of instance    
      tx_start => hdmi_tx_start,
      tx_iprg  => hdmi_tx_iprg,
      tx_data  => hdmi_tx_data,

      -- rx side of interface 
      rx_data  => hdmi_rx_data,
      data_rdy => hdmi_data_rdy
    );
    
  ssi_sata_inst : ssi
    PORT MAP (
      reset    => db_rst,
      clk      => clk,
      
      -- slave device interface
      ssi_rx   => sata_ssi_rx,
      ssi_tx   => sata_ssi_tx,
      ssi_strb => sata_ssi_strb,
      
      -- tx side of instance    
      tx_start => sata_tx_start,
      tx_iprg  => sata_tx_iprg,
      tx_data  => sata_tx_data,

      -- rx side of interface 
      rx_data  => sata_rx_data,
      data_rdy => sata_data_rdy
    );    

-- SSI state machine
  PROCESS(clk, db_rst)
  BEGIN
    IF ( db_rst = '1' ) THEN              
      ssi_state        <= IDLE;             
      tx_start         <= '0';
      ssi_data_ready_s <= '0';
      ssi_data_latch   <= (OTHERS => '0');
    ELSIF rising_edge(clk) THEN
      CASE ssi_state IS
        WHEN IDLE =>                            -- wait for start signal
          IF ( ssi_data_start_s = '1') THEN     -- from ARM
            ssi_data_ready_s <= '0';            -- reset FAB data ready signal
            tx_start         <= '1';            -- start transmission
            ssi_state        <= WAIT_FOR_DATA;
          ELSE
            ssi_state <= IDLE;
          END IF;
          
        -- wait while tx transmission starts
        WHEN WAIT_FOR_DATA =>         
          tx_start <= '0';
          IF ( data_rdy = '1' ) THEN      -- if data are ready
            ssi_state        <= IDLE;     
            ssi_data_ready_s <= '1';      -- set FAB data ready signal
            ssi_data_latch   <= rx_data;  -- record received data
          ELSE
            ssi_state <= WAIT_FOR_DATA;
          END IF;
      END CASE;
    END IF;
  END PROCESS;

  -- multiplexors for SSI interfaces
  -- TX start 
  sata_tx_start <= tx_start WHEN (mux_out = '1') ELSE
                   '0';
  hdmi_tx_start <= tx_start WHEN (mux_out = '0') ELSE
                   '0';
  --
  -- TX in progress
  tx_iprg <= sata_tx_iprg   WHEN (mux_out = '1') ELSE
             hdmi_tx_iprg;
  
  -- RX data (from DB)
  rx_data <= sata_rx_data   WHEN (mux_out = '1') ELSE
             hdmi_rx_data;
  -- data ready signal
  data_rdy <= sata_data_rdy WHEN (mux_out = '1') ELSE
              hdmi_data_rdy;  
 
  -- TX data (to DB)
  sata_tx_data <= tx_data;
  hdmi_tx_data <= tx_data;
 
  -- SSI transmission in progress
  ssi_in_prog_s  <= '1'     WHEN  ssi_state /= IDLE  ELSE '0';

  -- RNG alarm mux
  rng_alarm <= sata_rng_alarm WHEN (mux_out = '1') ELSE
               hdmi_rng_alarm;
  rng_alarm_irq   <=   rng_alarm; -- to interrupt MSS 

--***************************************************************************--
--*************** COMMAND execution state machine template ******************--
--***************************************************************************--
  PROCESS(clk, rst)
  BEGIN
    IF (rst = '1') THEN
      cmd_state       <= CMD_IDLE;
      mux_out         <= '0';
      requested_data  <= REQ_NONE;
      tx_data         <= (OTHERS => '0');
    ELSIF rising_edge(clk) THEN
      CASE cmd_state IS
        WHEN CMD_IDLE =>              -- command awaiting
          ssi_data_start_s <= '0';
          IF (new_cmd = '1') THEN
            cmd_state <= CMD_EXE;
          END IF;
        WHEN CMD_WAIT_THEN_IDLE =>
          -- Just stay here 1 clock cycle then switch to IDLE
          cmd_state <= CMD_IDLE;
        WHEN CMD_EXE  =>              -- command execution
          CASE cmd_reg IS
            WHEN CMD_OP_ST =>         -- operation is started from MSS
              cmd_state <= CMD_IDLE;  -- by inserting of this command
            WHEN CMD_SETDB1 =>        -- set daughter board 1 as active board
              mux_out   <= '0';       -- (HDMI)
              cmd_state <= CMD_IDLE;
            WHEN CMD_SETDB2 =>        -- set daughter board 2 as active board
              mux_out   <= '1';       -- (SATA)
              cmd_state <= CMD_IDLE;
            WHEN CMD_SSI_LOAD_HIGH => -- Load the high 32 bits of SSI word
              tx_data(63 DOWNTO 32) <= fabin_reg;
              cmd_state                  <= CMD_IDLE;
            WHEN CMD_SSI_LOAD_SEND_LOW =>      
              -- Send fabin contents to the DB
              tx_data(31 DOWNTO 0) <= fabin_reg;
              ssi_data_start_s  <= '1';
              cmd_state         <= CMD_IDLE; --CMD_WAIT_THEN_IDLE;  
            WHEN CMD_READ_SSI_HIGH => -- Read high 32 bits from SSI word
              requested_data <= REQ_SSI_HIGH_32;
              cmd_state      <= CMD_IDLE;
            WHEN CMD_READ_SSI_LOW =>  -- Read low 32 bits from SSI word
              requested_data <= REQ_SSI_LOW_32;
              cmd_state      <= CMD_IDLE;
            WHEN CMD_GET_TIME =>
              requested_data <= REQ_TIME;
              cmd_state <= CMD_IDLE;
            WHEN SET_N =>
              cmd_state <= CMD_IDLE;
            WHEN OTHERS => NULL;
          END CASE; 
        WHEN OTHERS => NULL;
      END CASE;
    END IF;
  END PROCESS;

  db_mux_out <= mux_out;  -- multiplexor output for connector choose
 
 
--***************************************************************************--


--***************************************************************************--
--************************ USER TEMPLATE DESIGN *****************************--
--***************************************************************************--

  PROCESS(clk, fab_rst)
  BEGIN
    IF (fab_rst = '1') THEN
      op_state      <= OP_IDLE;
      sp_data_vld_i <= '1';
      ram_in_vld    <= '0';
      ram_strt      <= '0';
      ram_rw        <= '0';
      ram_addr      <= (OTHERS => '0');
      ram_cnt       <= (OTHERS => '0');
      ram_in        <= (OTHERS => '0');
    ELSIF rising_edge(clk) THEN
      CASE op_state IS
        WHEN OP_IDLE =>
          IF(op_enable = '1') THEN         -- operation is started by start cmd
            ram_addr      <= wstart_reg;   -- start address of stream
            ram_cnt       <= wsize_reg;    -- size of required stream 
            ram_rw        <= '1';          -- set writing
            -- run ram interface 
            ram_strt      <= '1';
            op_state      <= OP_READ_DATA;
          END IF;    
        WHEN OP_READ_DATA =>
          ram_strt      <= '0';     
          IF (sp_data_vld = '1') THEN
            ram_in        <= sp_data;      -- insert data to ram interface
            ram_in_vld    <= '1';
          ELSE
            ram_in_vld    <= '0';
          END IF;

          IF(ram_ofw = '1')THEN            -- if ram interface overflowed
            op_state <= OP_OFLW;           -- stay in overflow state
          ELSIF(ram_bsy = '0')THEN         -- when enough of data was loaded
            op_state <= OP_IDLE;           -- set idle state
          END IF;
        WHEN OP_OFLW =>
          op_state <= OP_OFLW;
      END CASE;
    END IF;
  END PROCESS; 

  -- operation state which is read by MSS
  WITH op_state SELECT 
  state_reg <= OP_IDLE_STATE   WHEN OP_IDLE,
               OP_OFLW_STATE   WHEN OP_OFLW,
               OP_BUSY_STATE   WHEN OTHERS;
 
  -- acquisition enable state
  op_enable <= '1' WHEN ( (cmd_state = CMD_EXE  ) AND 
                          (cmd_reg   = CMD_OP_ST)     ) ELSE '0';
  
  
  -- Fabric status register
  fabst_reg(31 DOWNTO 28) <= MB_VER;     -- Motherboard version
  fabst_reg(27 DOWNTO 10) <= (OTHERS =>'0');
  fabst_reg(9)            <= d2_par_err;
  fabst_reg(8)            <= d1_par_err;
  fabst_reg(7)            <= '0';
  fabst_reg(6)            <= '0';
  fabst_reg(5)            <= '0';
  fabst_reg(4)            <= rng_alarm;  -- RNG alarm         -> 0 - NO ALARM
                                         --                   -> 1 -    ALARM
  fabst_reg(3)            <= '0';
  fabst_reg(2)            <= '0';
  
  -- ssi_in_prog_s     ->    '0' when device is idle or finished
  --                         '1' when device is currently transmiting data
  -- ssi_data_ready_s  ->    '0' when data are not available or inactual
  --                         '1' when ssi data are valid  
  fabst_reg(1 DOWNTO 0)  <= ssi_in_prog_s & ssi_data_ready_s;
  
  led_out <= rst OR db_rst;
    
  -- counter of time
  PROCESS(clk, fab_rst)
  BEGIN
    IF (fab_rst = '1') THEN
      ms_cnt   <= (OTHERS =>'0');
      time_cnt <= (OTHERS =>'0');
    ELSIF rising_edge(clk) THEN
      IF(state_reg = OP_BUSY_STATE)THEN
        IF(time_cnt = x"14438")THEN     -- set to 1 ms
          ms_cnt   <= ms_cnt + 1;
          time_cnt <= (OTHERS =>'0');
        ELSE
          time_cnt <= time_cnt + 1;
        END IF;
      ELSIF(op_enable = '1')THEN        -- when measurement is enabled
        time_cnt <= (OTHERS =>'0');     -- reset all signals
        ms_cnt   <= (OTHERS =>'0');
      END IF;
    END IF;
  END PROCESS;

  WITH requested_data SELECT
    fabout_reg   <= ms_cnt                        WHEN REQ_TIME,
                    ssi_data_latch(63 DOWNTO 32)  WHEN REQ_SSI_HIGH_32,
                    ssi_data_latch(31 DOWNTO 0)   WHEN REQ_SSI_LOW_32,
                    (OTHERS => '0')               WHEN OTHERS;


  mssio_in_reg(31 DOWNTO 0) <= (OTHERS =>'0');
  
  --
  fab_rst     <= mssio_out_reg(9); -- fabric_logic positive reset signal
  db_rst      <= mssio_out_reg(8); -- daughter board positive reset signal
  db_reset    <= db_rst;
  mux_sel     <= mssio_out_reg(0); -- output select of the daughter board
  
  -- 
--***************************************************************=************--

END arch_ctrl;
