-------------------------------------------------------------------------------
--
-- Design unit:   32-bit multiplexer and 1-bit serial output (to PP) 
--
-- File name:     mux32_serout.vhd
--
-- Description:   Reads 32-bit words comming from one of data connectors
--                (1 SATA) or (0 HDMI).
--                It converts 32-bit words to a serial bitstream sent to the 
--                post-processing block, using data2pp and data2pp_vld outputs.
--
-- System:        VHDL'93
--
-- Authors:       Marek Laban, Marcel Kleja, Ivan Pecar MICRONIC a.s.,  
--                Viktor Fischer, LabHC 
--
-- Copyright:     2017, MICRONIC a.s., LabHC
--
-- Created in the framework of HECTOR project (No. 644052).
--
-- Revision:      Version 1.00, May 2016
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
--
-------------------------------------------------------------------------------
LIBRARY ieee;

USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY mux32_serout IS
  PORT (
     -- Select data connector
    db_mux      : IN  STD_LOGIC;    -- Daughter board mux (1 SATA, 0 HDMI)
 
    -- SATA signals ----------------------------------
    sata_data   : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    sata_vld    : IN STD_LOGIC;

    -- HDMI signals ----------------------------------
    hdmi_data   : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    hdmi_vld    : IN STD_LOGIC;

    -- Output to the external RAM --------------------
    data_out    : OUT STD_LOGIC_VECTOR(31 DOWNTO 0); -- Data output - 32-bit word
    data_vld    : OUT STD_LOGIC                      -- Data valid signal (valid on 1) 
 
  );
END mux32_serout;

ARCHITECTURE rtl OF mux32_serout IS
  -- Data synchronization signals -------------------------------------------
  -- Output data synchronized to internal clock
  SIGNAL data_out_i    : STD_LOGIC_VECTOR (31 DOWNTO 0 );  
  -- Output data valid signal synchronized to internal clock  
  SIGNAL data_vld_i    : STD_LOGIC  := '0'; 

 

BEGIN
  -- Daughter board multiplexor choice
  WITH db_mux SELECT 
  data_out_i <= sata_data   WHEN '1',
                hdmi_data   WHEN OTHERS;

  WITH db_mux SELECT 
  data_vld_i <= sata_vld   WHEN '1',
                hdmi_vld   WHEN OTHERS;

  -- 32-bit data output to the external RAM
  data_out <= data_out_i;
  data_vld <= data_vld_i;
 
END rtl;
