-------------------------------------------------------------------------------
--
-- Design unit:   Serial to paralel converter 
--                 
--
-- File name:     ser2cnt.vhd
--
-- Description:   Reads bits comming from the TRNG core
--                Consists of two consecutive blocks:
--                  1) serial-to-parallel converters - converts received 
--                     bitstream to 32-bit data words. Data are valid on edges
--                     (rising and falling) of the validation signal.
--                     word is received.
--                  2) synchronization - read words from sampler and synchronizes
--                     them to system internal clocks.
--
-- System:        VHDL'93
--
-- Authors:       Marek Laban, MICRONIC a.s., LabHC
--                Viktor Fischer, LabHC   
--
-- Copyright:     2017, MICRONIC a.s., LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 2.00, September 2017
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
--                17/09/2017 - v2.00 Start/stop bits not used any more
--
-------------------------------------------------------------------------------
LIBRARY ieee;

USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY ser2cnt IS
  PORT (
  -- Motherboard internal signals
    clk        : IN  STD_LOGIC;    -- clock used for synchronization of data
    rst      : IN  STD_LOGIC;    -- global reset

  -- sampling clock of serial input (332 MHz)
    clk_smpl     : IN  STD_LOGIC;  -- clock input from daughter board
  
  -- Signals from the Daughter board
    ser_in     : IN  STD_LOGIC;  -- data input signal from daughter board
    ser_clk    : IN STD_LOGIC;   -- serial data clock (valid when state is changed) 
    -- Data output to the external RAM
    cnt_vld    : OUT STD_LOGIC;  -- data valid signal (valid on 1)                 
    cnt_out    : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
    --
    -- parity error detected
    parity_err : OUT STD_LOGIC
  );
END ser2cnt;

ARCHITECTURE ser2cnt OF ser2cnt IS
  -- sampled serial data 
  SIGNAL ser_in_i : STD_LOGIC;
  -- last value of clock signal
  SIGNAL ser_clk_i: STD_LOGIC;
  -- number of samples for ser_clk
  SIGNAL ser_clk_smpl: UNSIGNED(1 DOWNTO 0);
  -- synchronized serial data ready 
  SIGNAL ser_in_rdy  : STD_LOGIC;
  --
  TYPE out_sel_states IS (START_BIT, RECEIVE, PARITY_BIT, STOP_BIT);
  SIGNAL state : out_sel_states;  
  -- counter of transmitted bits
  SIGNAL bit_cnt : UNSIGNED(4 DOWNTO 0);
  -- counter word
  SIGNAL cnt_wrd  : STD_LOGIC_VECTOR(31 DOWNTO 0);
  -- latched word for next block
  SIGNAL cnt_wrd_i: STD_LOGIC_VECTOR(31 DOWNTO 0);
  -- latched by next block
  SIGNAL cnt_wrd_ii: STD_LOGIC_VECTOR(31 DOWNTO 0);
  -- parity bit calculation
  SIGNAL par_bit  : STD_LOGIC;
  --
  SIGNAL vld      : STD_LOGIC;                     -- auxilary valid signal

  -- data synchronization signals -------------------------------------------
  -- output data valid signal synchronized to internal clock  
  SIGNAL cnt_vld_i : STD_LOGIC  := '0'; 
  -- last validation value of the previous block
  SIGNAL last_vld   : STD_LOGIC;
  -- number of sampled values of valid signal from previous block
  SIGNAL smpl_time  : UNSIGNED (1 DOWNTO 0);


  
BEGIN

  PROCESS(clk_smpl, rst)
  BEGIN
    IF (rst = '1') THEN
      ser_in_i     <= '0';
      ser_clk_i    <= '0';
      ser_clk_smpl <= "00";
      ser_in_rdy   <= '0';
    ELSIF rising_edge(clk_smpl) THEN       -- fast clock
      IF(ser_clk_smpl > "01") AND (ser_clk /= ser_clk_i) THEN
        ser_in_rdy   <= '1';
        ser_in_i     <= ser_in;
        ser_clk_smpl <= "00";
        ser_clk_i    <= NOT ser_clk_i;
      ELSIF(ser_clk /= ser_clk_i) THEN
        ser_clk_smpl <= ser_clk_smpl + 1;
        ser_in_rdy   <= '0';
      ELSE
        ser_in_rdy   <= '0';
      END IF;
    END IF;
  END PROCESS;

  PROCESS(clk_smpl, rst)
  BEGIN
    IF (rst = '1') THEN
      bit_cnt    <= (OTHERS => '0');    
      vld        <= '0';
      cnt_wrd_i  <= (OTHERS =>'0');
      cnt_wrd    <= (OTHERS =>'0');
      par_bit    <= '0';
      parity_err <= '0';
      state      <= START_BIT;
    ELSIF rising_edge(clk_smpl) THEN       -- fast clock
      CASE state IS
        WHEN START_BIT =>                  -- Send start bit to TX line
          IF (ser_in_rdy = '1') AND (ser_in_i = '0') THEN-- wait for start bit 
            state   <= RECEIVE;
          END IF;
        WHEN RECEIVE =>                   -- Start to receive bits
          IF (ser_in_rdy = '1') THEN
            IF ( bit_cnt < "11111" ) THEN    -- Shift 32 bits
              bit_cnt              <= bit_cnt + 1; 
              cnt_wrd(31)          <= ser_in_i;
              par_bit              <= par_bit XOR ser_in_i;
              cnt_wrd(30 DOWNTO 0) <= cnt_wrd(31 DOWNTO 1);
            ELSE
              cnt_wrd(30 DOWNTO 0) <= cnt_wrd(31 DOWNTO 1);
              par_bit              <= par_bit XOR ser_in_i;
              bit_cnt              <= (OTHERS => '0');
              state                <= PARITY_BIT;
            END IF;
          END IF;
        WHEN PARITY_BIT =>
          IF (ser_in_rdy = '1') THEN -- wait for stop bit 
            IF (ser_in_i /= par_bit) THEN
              parity_err <= '1';
            END IF;
            cnt_wrd_i  <= cnt_wrd;
            vld        <= NOT vld  ;
            par_bit    <= '0';
            state      <= STOP_BIT;
          END IF;
        WHEN STOP_BIT =>                   -- Send stop bit
          IF (ser_in_rdy = '1') AND (ser_in_i = '1') THEN-- wait for STOP bit 
            state   <= START_BIT;
          END IF;
      END CASE;
    END IF;
  END PROCESS;
  

  -- synchronization of the received 32-bit data blocks to the internal clock --
  -- generation of data validation signal (data valid on its rising edge)
  PROCESS(clk, rst)
  BEGIN
    IF (rst = '1') THEN
      last_vld  <= '0';
      smpl_time <= (OTHERS =>'0');
      cnt_wrd_ii<= (OTHERS =>'0');
      cnt_vld_i<= '0';      
    ELSIF rising_edge(clk) THEN         -- clock from motherboard
      IF (last_vld /= vld) THEN 
        -- load data on output
        IF(smpl_time > "00") THEN
          smpl_time  <= (OTHERS =>'0'); -- refresh sample time
          last_vld   <= vld;            -- set new last valid value
          cnt_wrd_ii <= cnt_wrd_i;         -- set new data to output
          cnt_vld_i <= '1';            -- set output data valid signal
        ELSE
          smpl_time  <= smpl_time + 1;  -- increment sample time
          cnt_vld_i <= '0';
        END IF;
      ELSE   
        cnt_vld_i   <= '0';  -- data on output are valid during one clk period
      END IF;
    END IF;
  END PROCESS;

  -- data to output
  cnt_out <= cnt_wrd_ii;
  -- data valid signal to output
  cnt_vld <= cnt_vld_i;

END ser2cnt;
