-------------------------------------------------------------------------------
--
-- Design unit:   Serial to paralel converter 
--                 
--
-- File name:     ser2par.vhd
--
-- Description:   Reads bits comming from the TRNG core
--                Consists of two consecutive blocks:
--                  1) serial-to-parallel converters - converts received 
--                     bitstream to 32-bit data words. Data are valid on edges
--                     (rising and falling) of the validation signal.
--                     word is received.
--                  2) synchronization - read words from sampler and synchronizes
--                     them to system internal clocks.
--
-- System:        VHDL'93
--
-- Authors:       Marek Laban, MICRONIC a.s., LabHC
--                Viktor Fischer, LabHC   
--
-- Copyright:     2017, MICRONIC a.s., LabHC
--
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version 2.00, September 2017
-- History:       16/05/2016 - v1.00 Created (ML, MIC)
--                17/09/2017 - v2.00 Start/stop bits not used any more
--
-------------------------------------------------------------------------------
LIBRARY ieee;

USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY ser2par IS
  PORT (
  -- Motherboard internal signals
    clk        : IN  STD_LOGIC;    -- clock used for synchronization of data
    reset      : IN  STD_LOGIC;    -- global reset

  -- Signals from the Daughter board
    clk_in     : IN  STD_LOGIC;    -- clock input from daughter board
    data_in    : IN  STD_LOGIC;    -- data input signal from daughter board

    -- Data output to the external RAM
    data_vld   : OUT STD_LOGIC;    -- data valid signal (valid on 1)                 
    data_out   : OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
  );
END ser2par;

ARCHITECTURE ser2par OF ser2par IS
  -- internal signal and registers declaration
  SIGNAL cnt_bits : UNSIGNED(5 DOWNTO 0);          -- bits counter
  SIGNAL vld      : STD_LOGIC;                     -- auxilary valid signal
  --
  SIGNAL data_i   : STD_LOGIC_VECTOR(31 DOWNTO 0); -- data output register

  -- shift register for received data
  SIGNAL shreg    : STD_LOGIC_VECTOR (31 DOWNTO 0 ) := ( OTHERS => '0' );  

  -- data synchronization signals -------------------------------------------
  -- output data synchronized to internal clock
  SIGNAL data_out_i : STD_LOGIC_VECTOR (31 DOWNTO 0 ) := ( OTHERS => '0' );  
  -- output data valid signal synchronized to internal clock  
  SIGNAL data_vld_i : STD_LOGIC  := '0'; 
  -- last validation value of the previous block
  SIGNAL last_vld   : STD_LOGIC;
  -- number of sampled values of valid signal from previous block
  SIGNAL smpl_time  : UNSIGNED (1 DOWNTO 0);

BEGIN
-- Process synchronized with the daughterboard clock
-- Counter of 32 serial input bits
-- Serial-to-parallel converter
  PROCESS(clk_in, reset)
  BEGIN
    IF (reset = '1') THEN
      cnt_bits  <= (OTHERS => '0');    
      vld       <= '0';
      data_i    <= (OTHERS =>'0');
      shreg     <= (OTHERS =>'0');
    ELSIF rising_edge(clk_in) THEN       -- clock from daughter board  
      IF(cnt_bits = "011111")THEN        -- every 32 bit set new word to output
        vld       <= NOT vld  ;          -- invert output strobe signal
--        data_i    <= shreg (30 DOWNTO 0) & data_in;  -- set new word
        data_i    <= shreg;              -- save new word
        cnt_bits  <= "000000";
      ELSE
--        shreg     <= shreg (30 DOWNTO 0) & data_in;  -- insert new bit
        cnt_bits  <= cnt_bits + 1;       -- received bits counter 
      END IF;
      shreg     <= shreg (30 DOWNTO 0) & data_in;  -- insert new bit
    END IF;
  END PROCESS;
  

  -- synchronization of the received 32-bit data blocks to the internal clock --
  -- generation of data validation signal (data valid on its rising edge)
  PROCESS(clk, reset)
  BEGIN
    IF (reset = '1') THEN
      last_vld  <= '0';
      smpl_time <= (OTHERS =>'0');
      data_out_i<= (OTHERS =>'0');
      data_vld_i<= '0';      
    ELSIF rising_edge(clk) THEN         -- clock from motherboard
      IF (last_vld /= vld) THEN 
        -- load data on output
        IF(smpl_time > "00") THEN
          smpl_time  <= (OTHERS =>'0'); -- refresh sample time
          last_vld   <= vld;            -- set new last valid value
          data_out_i <= data_i;         -- set new data to output
          data_vld_i <= '1';            -- set output data valid signal
        ELSE
          smpl_time  <= smpl_time + 1;  -- increment sample time
          data_vld_i <= '0';
        END IF;
      ELSE   
        data_vld_i   <= '0';  -- data on output are valid during one clk period
      END IF;
    END IF;
  END PROCESS;

  -- data to output
  data_out <= data_out_i;
  -- data valid signal to output
  data_vld <= data_vld_i;

END ser2par;
