-------------------------------------------------------------------------------
--
-- Design unit:   SSI communication module
--                 
--
-- File name:     ssi.vhd
--
-- Description:   Transfer data between motherboard and daughter board using 
--                three wires (strb, tx, rx). Protocol uses start and stop bit
--                in order to allign 64 bit words.
-- 
--
-- System:        VHDL'93
--
-- Author:        Oto Petura, Marek Laban, Viktor Fischer LabHC
--
-- Copyright:     MICRONIC a.s., LabHC
--  
-- Created in frame of HECTOR project (No. 644052).
--
-- Revision:      Version v5.00, February 2017
-- History:       17/02/2017 - v5.00 Sctructure modified (ML, MIC)
--                30/05/2017 - v8.00 Clock signal changed to strobe
--                                   RX and TX componnents included here
--
-------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

ENTITY ssi IS
  PORT (
    reset    : IN  STD_LOGIC;     -- Positive reset
    clk      : IN  STD_LOGIC;     -- Input clock
    --
    -- data input/output from device
    ssi_rx   : IN  STD_LOGIC;     -- RX serial line
    ssi_tx   : OUT STD_LOGIC;     -- TX serial line
    ssi_strb : IN  STD_LOGIC;     -- strobe signal 
    --
    -- tx side ------------------------------------------  
    -- Parallel data input
    tx_data  : IN  STD_LOGIC_VECTOR ( 63 DOWNTO 0 ); 
    tx_iprg  : OUT STD_LOGIC;     -- transfer in progress
    tx_start : IN  STD_LOGIC;     -- when asserted, data are transmitted by ssi
    --
    -- rx side ------------------------------------------
    -- Parallel data output
    rx_data  : OUT STD_LOGIC_VECTOR ( 63 DOWNTO 0 );
    data_rdy : OUT STD_LOGIC      -- RX data ready
  );
END ssi;

ARCHITECTURE ssi_arch OF ssi IS
  SIGNAL ssi_strb_i       : STD_LOGIC; -- sampled ssi strb signal
  SIGNAL ssi_strb_ii      : STD_LOGIC; -- delayed sampled ssi strb signal
  SIGNAL ssi_rx_i         : STD_LOGIC; -- sampled ssi RX data
  SIGNAL rising_ssi_strb  : STD_LOGIC; -- rising edge of strb signal detected
  SIGNAL falling_ssi_strb : STD_LOGIC; -- falling edge of strb signal detected
  
  -- TX state machine states
  TYPE   tx_state_type IS (IDLE, SEND_DATA);  
  SIGNAL tx_state   : tx_state_type;
  
  -- TX transmit shift register (64 bits wide with start and stop bit)
  SIGNAL tx_reg     : STD_LOGIC_VECTOR(65 DOWNTO 0);
  SIGNAL tx_cnt     : UNSIGNED (6 DOWNTO 0);  -- counter of transmitted bits
  --
  -- TX state machine states  
  TYPE   rx_state_type IS (WAIT_START, RECV_DATA, WAIT_STOP);
  SIGNAL rx_state : rx_state_type;
  
  -- TX transmit shift register (64 bits wide)  
  SIGNAL rx_reg     : STD_LOGIC_VECTOR(63 DOWNTO 0); -- received rx data 
  SIGNAL rx_cnt     : UNSIGNED        (5 DOWNTO 0);  -- counter of received bits

BEGIN
  -- strobe signal sampling
  PROCESS(clk)
  BEGIN
    IF rising_edge(clk) THEN
      --      
      ssi_strb_i   <= (NOT ssi_strb);  -- sampling of strobe signal
      ssi_strb_ii  <= ssi_strb_i;      -- delayed strobe signal
      -- 
      ssi_rx_i     <= ssi_rx;          -- sampled data
    END IF;
  END PROCESS;
  --
  falling_ssi_strb  <=      ssi_strb_i  AND (NOT ssi_strb_ii);
  rising_ssi_strb   <= (NOT ssi_strb_i) AND      ssi_strb_ii;
  
  -- TX state machine
  PROCESS( reset, clk )
  BEGIN 
    IF ( reset = '1' ) THEN
      tx_state <= IDLE;
      ssi_tx   <= '1';
      tx_cnt   <= (OTHERS => '0');
      tx_reg   <= (OTHERS => '0');
    ELSIF rising_edge( clk ) THEN
      CASE tx_state IS
        WHEN IDLE =>                       -- wait for start signal
          IF (tx_start = '1') THEN         
            tx_reg <= '0' & tx_data & '1'; -- load tx word and inserts 
            --                             -- start and stop bit
            tx_state   <= SEND_DATA;       -- start transmitting
          END IF;
        WHEN SEND_DATA =>
          IF (rising_ssi_strb = '1') THEN  -- when rising edge detected
            ssi_tx  <= tx_reg (65);        -- send one bit and shift register
            tx_reg  <= tx_reg (64 DOWNTO 0) & '0';

            IF(tx_cnt = X"41") THEN        -- when entire register was sent
              tx_cnt   <= (OTHERS => '0'); -- reset bit counter
              tx_state <= IDLE;            -- go to idle state
            ELSE                    
              tx_cnt <= tx_cnt + 1;        -- counts transmitted bits 
            END IF;
          END IF;
      END CASE;
    END IF;
  END PROCESS;  
  -- transmission in progress signal
  tx_iprg <= '1' WHEN (tx_state = SEND_DATA) ELSE '0';
  --
  -- RX state machine
  PROCESS( reset, clk )
  BEGIN
    IF ( reset = '1' ) THEN
      rx_state <= WAIT_START;
      rx_reg   <= (OTHERS => '0');
      rx_cnt   <= (OTHERS => '0');
      data_rdy <= '0';
    ELSIF rising_edge( clk ) THEN
      CASE rx_state IS
        WHEN WAIT_START =>        -- wait for start bit      
          data_rdy <= '0';        -- reset data ready signal
          --
          -- when falling edge and start bit detected -> receive data
          IF ((falling_ssi_strb = '1') AND (ssi_rx_i = '0')) THEN
            rx_state <= RECV_DATA;
          END IF;
        --  
        WHEN RECV_DATA =>
          IF (falling_ssi_strb = '1') THEN               -- every falling edge 
            rx_reg(0)           <= ssi_rx_i;             -- one bit is inserted 
            rx_reg(63 DOWNTO 1) <= rx_reg(62 DOWNTO 0);  -- to shift register
            -- 
            IF(rx_cnt = "111111") THEN       -- when register is full
              rx_cnt    <= (OTHERS => '0');  -- reset counter
              rx_state  <= WAIT_STOP;        -- wait stop bit
            ELSE                   
              rx_cnt    <= rx_cnt + 1;       -- else count received bits
            END IF;
          END IF;
        WHEN WAIT_STOP =>           
          -- waits for falling edge and stop bit
          IF((falling_ssi_strb = '1') AND (ssi_rx_i = '1')) THEN
            data_rdy <= '1';
            rx_state <= WAIT_START;
          END IF;          
      END CASE;
    END IF;
  END PROCESS;    
  
  rx_data <= rx_reg;
    
END ssi_arch;
