# Load the HECTOR tcl extension
source HECTOR_data_acq.tcl

# Size of data to retrieve
set filesize 128
# Set speed of generator
set n_value 1000000

# Set the variables for commands
set load_n_value 12    
set load_ssi_high_32 8
set send_ssi_word 6
set read_ssi_high_32 9
set read_ssi_low_32 10
set read_acquisition_time 7

# Open the serial device
set dev [openDevice]

for {set i 0} {$i < 1} {incr i} {
  
# Set timestamp for file name
  set filenameTimestamp [clock format [clock seconds] -format "%Y%m%d%H%M%S"]
  puts "Measurement # $i (timestamp: $filenameTimestamp)"

# Open log file
  set logFile [open "data/log_$filenameTimestamp\_$i.txt" "w"]
  
# Reset motherboard
  softReset $dev  
  sendFabricReset $dev 
# Select the daughter board connection (1 = HDMI, 2 = SATA)
  selectDaughterBoard $dev 2  
  
# Reset daughterboard
  sendDaughterReset $dev  

# Load N value to motherboard
  sendFabricCommand $dev $load_n_value $n_value  

# Load N value to daughter board
  sendFabricCommand $dev $load_ssi_high_32 $n_value    
  sendFabricCommand $dev $send_ssi_word 3     

# Select RNG data
  sendFabricCommand $dev $load_ssi_high_32 0
  sendFabricCommand $dev $send_ssi_word 6


 
  createFileSystem $dev   
 
 
# Begin data acquisition
#     We must provide the filename that will be assigned
#     to acquired data file. Last parameter sets the size
#     of the file in bytes.
  beginAcquisition $dev ACQ.DAT $filesize      

# Wait until the acquisition is finished
  set err [getStatus $dev ]
  while {[lindex $err 2] != 0} {
    set err [getStatus $dev ]

# Print progress
    puts -nonewline "[lindex $err 1] %\r"
    flush stdout
  }
  puts {}
  flush stdout
  
# Get TRNG status
  sendFabricCommand $dev $load_ssi_high_32 0 
  sendFabricCommand $dev $send_ssi_word 0   
  set ret [getStatus $dev]
  while { [getBitValue [fromHEX [lindex $ret 4]] 0] == 2 } {
    set ret [getStatus $dev]
  }
  puts {}

  # Load status word ######################################
  set fab_stat [lindex $ret 4]

  set par_err       [fromHEX [string range $fab_stat 5 5]]

  
  
# Load the daughter response
  set high_32 [lindex [sendFabricCommand $dev $read_ssi_high_32 0] 1]
  set low_32 [lindex [sendFabricCommand $dev $read_ssi_low_32 0] 1]

  #puts "$high_32$low_32"
  # puts $low_32
  # puts {}

  set n_daugh [fromHEX [string range $high_32  0 7]]
  set Ver_h [fromHEX [string range $low_32 0 1]]
  set Ver_l [fromHEX [string range $low_32 2 3]]

  set var   [fromHEX [string range $low_32  7 7]]

  set ovrflw_cnt   [expr [expr $var >> 3] & 1] 
  set var_finished [expr [expr $var >> 2] & 1] 
  set ovrflw_acc   [expr [expr $var >> 1] & 1] 
  set ovrflw_pwr2  [expr [expr $var     ] & 1] 

  if {$par_err > 0} {
     puts "\t!!!!Parity error detected!!!!     reg:  $par_err"    
     puts $logFile "\t!!!!Parity error detected!!!!     reg:  $par_err"    
  }


  puts "\tN value (generator speed):    $n_daugh "
  puts "\tDaughter board version:       $Ver_h.$Ver_l"
  puts " "
  puts $logFile "\tN value (generator speed):            $n_daugh "
  puts $logFile "\tRNG version                           $Ver_h.$Ver_l"
  puts $logFile " "

  set time [lindex [sendFabricCommand $dev $read_acquisition_time 0] 1]
  puts "\tFilesize               [expr ($filesize)] Bytes"
  puts "\tAcquisition time       [fromHEX $time] ms"
  puts "\tBitrate                [expr ($filesize * 8.0) / ([fromHEX $time] / 1.0)] kbits/s"
  puts $logFile "\tFilesize               [expr ($filesize)] Bytes"
  puts $logFile "\tAcquisition time       [fromHEX $time] ms"
  puts $logFile "\tBitrate                [expr ($filesize * 8.0) / ([fromHEX $time] / 1.0)] kbits/s"
  puts {}

# Mount the disk to make it available for the PC
  mountDisk $dev

# Automatically detect the HECTOR disk
  set hectorDrive [findHECTOR 15]

  if { $hectorDrive == 0 } {
    puts "Hector disk drive not found. You will have to search for it manually."
    return 1
  }

# Check for the mountpoint
  if { [string length [lindex $hectorDrive 2]] == 0 } {
    puts "Hector disk drive is not mounted. Please mount the device [lindex $hectorDrive 0]."
    return 1
  }

# Copy the data from HECTOR disk
  file copy -force "[lindex $hectorDrive 2]/ACQ.DAT" "data/acq_$filenameTimestamp\_$i.bin"
  # Create filesystem for data
  createFileSystem $dev
  #exec cmd /c mountvol [lindex $hectorDrive 2] /P
}

close $logFile

# Close the connection to the board
disconnect $dev
